require 'cgi'
require 'delegate'
require 'json'
require 'net/http'
require 'timeout'
require 'uri'

module KOAUtils
  class TimeoutResponse

    def successful?
      false
    end

    def code
      '0'
    end

    def self.body_permitted?
      false
    end

    def body
      nil
    end
  end

  class ResponseDecorator < SimpleDelegator
    def successful?
      code.to_i / 100 == 2
    end
  end

  module Request
    def self.make(opts)
      opts[:tries] ||= 1
      opts[:timeout] ||= 10

      opts[:url] = build_url(opts)
      opts[:body] = build_body(opts)

      execute(opts)
    end

    private

    def self.execute(opts)
      opts[:tries].times do

        begin
          ::Timeout::timeout(opts[:timeout]) do
            return ResponseDecorator.new(net_request(opts))
          end
        rescue ::Timeout::Error
        end

      end
      KOAUtils::TimeoutResponse.new
    end

    def self.net_request(opts)
      uri = ::URI.parse(opts[:url])
      http = ::Net::HTTP.new(uri.host, uri.port)
      http.use_ssl = uri.scheme == "https"
      request = request_factory(opts[:type]).new(uri.request_uri)

      request.basic_auth uri.user, uri.password if uri.user
      request.basic_auth opts[:auth][:user], opts[:auth][:pass] if opts[:auth]

      if opts[:type] == :post || opts[:type] == :put
        request.add_field('Content-Type', 'application/json')
        request.add_field('Content-Length', opts[:body].size)
        request.body = opts[:body]
      end

      http.request(request)
    end

    def self.request_factory(type)
      return ::Net::HTTP::Post if type == :post
      return ::Net::HTTP::Put if type == :put
      return ::Net::HTTP::Delete if type == :delete
      return ::Net::HTTP::Get if type == :get
    end

    def self.build_body(opts)
      body = ""
      if opts[:type] == :post || opts[:type] == :put
        body = ::JSON.dump(opts[:data]) if opts[:data]
      end
      body
    end

    def self.build_url(opts)
      url = opts[:url]
      if opts[:type] == :get || opts[:type] == :delete
        url += "?"+hash_to_query(opts[:data]) if opts[:data]
      end
      url
    end

    def self.hash_to_query(hash)
      hash.map{|k,v| "#{CGI.escape(k.to_s)}=#{CGI.escape(v.to_s)}"}.join("&")
    end
  end
end
